//+------------------------------------------------------------------+
//|                                                       MACENA.mq5 |
//|              Copyright Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

#include <ExpertAdvisor.mqh>

input int    SL        =  50; //  
input int    TP        = 100; //  
input int    TS        =  50; //   
input int    FastEMA   =  15; //  EMA
input int    SlowEMA   =  26; //  EMA
input int    MACD_SMA  =   1; //   MACD
input int    EnvelPer  =  20; //  Envelopes
input double EnvelDev  = 0.4; //  Envelopes
input double Risk      = 0.1; // 
//---
class CMyEA : public CExpertAdvisor
  {
protected:
   double            m_risk;          //  
   int               m_sl;            // 
   int               m_tp;            // 
   int               m_ts;            // 
   int               m_pFastEMA;      //  EMA
   int               m_pSlowEMA;      //  EMA
   int               m_pMACD_SMA;     //   MACD
   int               m_EnvelPer;      //  Envelopes     
   double            m_EnvelDev;      //  Envelopes     
   int               m_hmacd;         //   CD
   int               m_henvel;        //   Envelopes   
public:
   void              CMyEA();
   void             ~CMyEA();
   virtual bool      Init(string smb,ENUM_TIMEFRAMES tf); // 
   virtual bool      Main();                              //  
   virtual void      OpenPosition(long dir);              //    
   virtual void      ClosePosition(long dir);             //    
   virtual long      CheckSignal(bool bEntry);            //  
  };
//------------------------------------------------------------------	CMyEA
void CMyEA::CMyEA() { }
//------------------------------------------------------------------	~CMyEA
void CMyEA::~CMyEA()
  {
   IndicatorRelease(m_hmacd); //   CD
   IndicatorRelease(m_henvel);//   Envelopes
  }
//------------------------------------------------------------------	Init
bool CMyEA::Init(string smb,ENUM_TIMEFRAMES tf)
  {
   if(!CExpertAdvisor::Init(0,smb,tf)) return(false);                                //   
                                                                                     //  
   m_risk=Risk;
   m_tp=TP;
   m_sl=SL;
   m_ts=TS;
   m_pFastEMA=FastEMA;
   m_pSlowEMA=SlowEMA;
   m_pMACD_SMA=MACD_SMA;
   m_EnvelPer = EnvelPer;
   m_EnvelDev = EnvelDev;
   m_hmacd=iMACD(m_smb,PERIOD_D1,m_pFastEMA,m_pSlowEMA,m_pMACD_SMA,PRICE_CLOSE);     //   CD
   m_henvel=iEnvelopes(m_smb,PERIOD_H1,m_EnvelPer,0,MODE_SMA,PRICE_CLOSE,m_EnvelDev);//   Envelopes
   if(m_hmacd==INVALID_HANDLE ||m_henvel==INVALID_HANDLE ) return(false);            //    
   m_bInit=true; return(true);                                                       // " "
  }
//------------------------------------------------------------------	Main
bool CMyEA::Main() //  
  {
   if(!CExpertAdvisor::Main()) return(false); //    

   if(Bars(m_smb,m_tf)<=100) return(false);   //   

   if(!CheckNewBar()) return(true);           //   

                                              //   
   long dir;
   dir=ORDER_TYPE_BUY;
   OpenPosition(dir); ClosePosition(dir); TrailingPosition(dir,m_ts);
   dir=ORDER_TYPE_SELL;
   OpenPosition(dir); ClosePosition(dir); TrailingPosition(dir,m_ts);

   return(true);
  }
//------------------------------------------------------------------	OpenPos
void CMyEA::OpenPosition(long dir)
  {
//   ,  
   if(PositionSelect(m_smb)) return;
//      
   if(dir!=CheckSignal(true)) return;
   double lot=CountLotByRisk(m_sl,m_risk,0);
//    
   if(lot<=0) return;
//  
   DealOpen(dir,lot,m_sl,m_tp);
  }
//------------------------------------------------------------------	ClosePos
void CMyEA::ClosePosition(long dir)
  {
//   ,  
   if(!PositionSelect(m_smb)) return;
//     
   if(dir!=PositionGetInteger(POSITION_TYPE)) return;
//        
   if(dir!=CheckSignal(false)) return;
//  
   m_trade.PositionClose(m_smb,1);
  }
//------------------------------------------------------------------	CheckSignal
long CMyEA::CheckSignal(bool bEntry)
  {
   double macd[4],   //    MACD
   env1[3],   //     Envelopes
   env2[3];   //     Envelopes

   MqlRates rt[3];   //     3-  

   if(CopyRates(m_smb,m_tf,0,3,rt)!=3) //      3-  
     {
      Print("CopyRates ",m_smb,"   ");
      return(WRONG_VALUE);
     }
//     
   if(CopyBuffer(m_hmacd,0,0,4,macd)<4 || CopyBuffer(m_henvel,0,0,2,env1)<2 || CopyBuffer(m_henvel,1,0,2,env2)<2)
     {
      Print("CopyBuffer -  ");
      return(WRONG_VALUE);
     }
//   MACD       Envelopes
   if(rt[1].open<env2[1] && rt[1].close>env2[1] && macd[1]<macd[2] && macd[2]<macd[3])
      return(bEntry ? ORDER_TYPE_BUY:ORDER_TYPE_SELL); //   
//   MACD       Envelopes   
   if(rt[1].open>env1[1] && rt[2].close<env1[1] && macd[1]>macd[2] && macd[2]>macd[3])
      return(bEntry ? ORDER_TYPE_SELL:ORDER_TYPE_BUY); //   

   return(WRONG_VALUE); //   
  }

CMyEA ea; //  
//------------------------------------------------------------------	OnInit
int OnInit()
  {
   ea.Init(Symbol(),Period()); //  

                               //  
// ea.Init(Symbol(), PERIOD_M5); //   
// ea.Init("USDJPY", PERIOD_H2); //     

   return(0);
  }
//------------------------------------------------------------------	OnDeinit
void OnDeinit(const int reason) { }
//------------------------------------------------------------------	OnTick
void OnTick()
  {
   ea.Main(); //   
  }
//+------------------------------------------------------------------+
